/* -*- C -*- */
/* 

   Yorick interface definitions
   for Claus Schroeter GPIB library 

   by Alexey Goldin <alexey@oddjob.uchicago.edu>

   It is not finished and will never be, but it is usable.

To build: yorick -batch make.i ygpib gpib.i  + matrix.i fft.i 
Then: make 
Then copy gpib.i to startup directory 
(like /usr/local/share/yorick/1.4/startup)

*/

/* MAKE-INSTRUCTIONS
SRCS = ygpib_extra.c
LIB = ygpib 
DEPLIBS = gpib  fl
*/

/* All these functions are described in ib.h */
extern ibsta;
reshape, ibsta, int;

extern ibcnt;
reshape, ibcnt, int;

extern iberr;
reshape, iberr, int;



/* IBSTA status bits (returned by all functions) */

/* Error bits --- in gpib_user.h  */
ERR      = (1 << 15);      /* Function call terminated on error */
TIMO     = (1 << 14);      /* Time limit on I/O or wait function 
			   exceeded */
END      = (1 << 13);      /* EOI terminated the ibrd call */
SRQI     = (1 << 12);      /* SRQ is asserted */
RQS      = (1 << 11);      /* Device requesting Service */
CMPL     = (1 <<  8);      /* I/O is complete (should always be set) */
CIC      = (1 <<  5);      /* GPIB interface is Controller-in-Charge */
ATN      = (1 <<  4);      /* Attention is asserted */
TACS     = (1 <<  3);      /* GPIB interface is addressed as Talker */
LACS     = (1 <<  2);      /* GPIB interface is addressed as Listener */

func explain_status(status) {
  ret = [];
  if(status & ERR)    grow, ret,  "Function call terminated on error";
  if(status &  TIMO)  grow, ret, "Time limit on I/O or wait function exceeded"
  if(status &  END)   grow, ret, "EOI terminated the ibrd call";
  if(status &  SRQI)  grow, ret, "SRQ is asserted";
  if(status &  RQS)   grow, ret, "Device requesting Service";
  if(status &  CMPL)  grow, ret, "I/O is complete (should always be set)";
  if(status &  CIC)   grow, ret, "GPIB interface is Controller-in-Charge";
  if(status &  ATN)   grow, ret, "Attention is asserted";
  if(status &  TACS)  grow, ret, "GPIB interface is addressed as Talker";
  if(status &  LACS)  grow, ret, "GPIB interface is addressed as Listener";
  return ret;
}


/* IBERR error codes */
				/*  */
EDVR =    0		; /* system error */
ECIC =    1		; /* not CIC */
ENOL =    2		; /* no listeners */
EADR =    3		; /* CIC and not addressed before I/O */
EARG =    4		; /* bad argument to function call */
ESAC =    5		; /* not SAC */
EABO =    6		; /* I/O operation was aborted */
ENEB =    7		; /* non-existent board (GPIB interface offline) */
EDMA =    8		; /* DMA hardware error detected */
EBTO =    9		; /* DMA hardware uP bus timeout */
EOIP =   10		; /* new I/O attempted with old I/O in progress  */
ECAP =   11		; /* no capability for intended opeation */
EFSO =   12		; /* file system operation error */
EOWN =   13		; /* shareable board exclusively owned */
EBUS =   14		; /* bus error */
ESTB =   15		; /* lost serial poll bytes */
ESRQ =   16		; /* SRQ stuck on */

ECFG =   17              ; /* Config file not found */
EPAR =   18              ; /* Parse Error in Config */
ETAB =   19              ; /* Table Overflow */
ENSD =   20              ; /* Device not found */
ENWE =   21              ; /* Network Error */
ENTF =   22              ; /* Nethandle-table full */
EMEM =   23              ; /* Memory allocation Error */


func explain_error(code)  {
  local errtable;
  errtable = [ 
	    "system error",
	    "not CIC",
	    "no listeners",
	    "CIC and not addressed before I/O",
	    "bad argument to function call",
	    "not SAC",
	    "I/O operation was aborted",
	    "non-existent board (GPIB interface offline)",
	    "DMA hardware error detected",
	    "DMA hardware uP bus timeout",
	    "new I/O attempted with old I/O in progress ",
	    "no capability for intended opeation",
	    "file system operation error",
	    "shareable board exclusively owned",
	    "bus error",
	    "lost serial poll bytes",
	    "SRQ stuck on",
	    "Config file not found",
	    "Parse Error in Config",
	    "Table Overflow",
	    "Device not found",
	    "Network Error",
	    "Nethandle-table full",
	    "Memory allocation Error"
  ];

  if(code <=24) {
    return errtable(code+1);
  } 
  else
    {
      return "Unknown error";
    }
}
 /* Timeout values and meanings */
				
TNONE =   0		; /* Infinite timeout (disabled)     */
T10us =   1		; /* Timeout of 10 usec (ideal)      */
T30us =   2		; /* Timeout of 30 usec (ideal)      */
T100us=   3		; /* Timeout of 100 usec (ideal)     */
T300us=   4		; /* Timeout of 300 usec (ideal)     */
T1ms  =   5		; /* Timeout of 1 msec (ideal)       */
T3ms  =   6		; /* Timeout of 3 msec (ideal)       */
T10ms =   7		; /* Timeout of 10 msec (ideal)      */
T30ms =   8		; /* Timeout of 30 msec (ideal)      */
T100ms=   9		; /* Timeout of 100 msec (ideal)     */
T300ms=  10		; /* Timeout of 300 msec (ideal)     */
T1s   =  11		; /* Timeout of 1 sec (ideal)        */
T3s   =  12		; /* Timeout of 3 sec (ideal)        */
T10s  =  13		; /* Timeout of 10 sec (ideal)       */
T30s  =  14		; /* Timeout of 30 sec (ideal)       */
T100s =  15		; /* Timeout of 100 sec (ideal)      */
T300s =  16		; /* Timeout of 300 sec (ideal)      */
T1000s=  17		; /* Timeout of 1000 sec (maximum)   */


/* End-of-string (EOS) modes for use with ibeos */

 REOS =    0x04		; /* Terminate reads on EOS	*/
 XEOS =    0x08		; /* Set EOI with EOS on writes	*/
 BIN  =    0x10		; /* Do 8-bit compare on EOS	*/

 EOSM =	(REOS | XEOS | BIN);


/* GPIB Bus Control Lines bit vector */

 BUS_DAV =	 0x0100		; /* DAV  line status bit */
 BUS_NDAC= 0x0200		; /* NDAC line status bit */
 BUS_NRFD= 0x0400		; /* NRFD line status bit */
 BUS_IFC=	 0x0800		; /* IFC  line status bit */
 BUS_REN=	 0x1000		; /* REN  line status bit */
 BUS_SRQ=	 0x2000		; /* SRQ  line status bit */
 BUS_ATN=	 0x4000		; /* ATN  line status bit */
 BUS_EOI=	 0x8000		; /* EOI  line status bit */


/* Possible GPIB command messages */

 GTL=	0x1	; /* go to local			*/
 SDC=	0x4	; /* selected device clear 	*/
 PPC=	0x5	; /* parallel poll configure	*/
 GET=	0x8	; /* group execute trigger 	*/
 TCT=	0x9	; /* take control 		*/

 LLO=	0x11	; /* local lockout		*/
 DCL=	0x14	; /* device clear 		*/
 PPU=	0x15	; /* parallel poll unconfigure 	*/
 SPE=	0x18	; /* serial poll enable 		*/
 SPD=	0x19	; /* serial poll disable 		*/
 UNL=	0x3F	; /* unlisten 			*/
 UNT=	0x5F	; /* untalk 			*/
 PPE=	0x60	; /* parallel poll enable (base)	*/
 S=	0x08	; /* parallel poll sense bit	*/

extern ibcac;
/* PROTOTYPE 
   int ibcac(int ud, int v);
   */
/* DOCUMENT    int ibcac(int ud, int sync);
   makes board the active controller
   sync = 0 board take control  asynchronously
   sync != 0 waits for transfers to finish.
   
 */

extern  ibclr; 
/* PROTOTYPE 
int ibclr(int ud); */
/* DOCUMENT 
   ibclr(dev) sends Selected Device Clear message
*/

extern ibcmd_raw;
/* PROTOTYPE 
   int ibcmd(int ud,string cmd,long cnt);
 */

func ibcmd(ud, cmd) {
  /* DOCUMENT ibcmd(ud,cmd)
   Sends GPIB multiline commands. Do not confuse with ibwrt!!!
   Does not send commands to devices.
   */
  return ibcmd_raw(ud, cmd, strlen(cmd));
}



extern  ibfind_raw;
/* PROTOTYPE  
int ibfind(string dev); 
*/


extern ibeos;			
/* PROTOTYPE   int ibeos(int ud, int v);*/
/* DOCUMENT int ibeos(int dev, int eos) 
   SetEnd-Of-String termination mode. */

extern ibeot;			
/* PROTOTYPE   int ibeot(int ud, int v);*/
/* DOCUMENT int ibeot(int dev, int eot) 
   Changes assertion of EOI on write. */

extern ibgts;			
/* PROTOTYPE   int ibgts(int ud, int v);*/
/* DOCUMENT int ibeot(int dev, int handshake) 
   Puts an active controller in Standby mode. */


extern iblines_raw;
/* PROTOTYPE   int iblines(int ud, int array v);*/

func iblines(board) {
/* DOCUMENT int iblines(int board)
   returns status of GPIB lines 
   High byte    15  14  13  12  11  10   9    8
   (asserted) 
               EOI  ATN SRQ REN IFC NRFD NDAC DAV
   Low byte      7    6   5   4   3    2    1   0
   (capabilities)
*/
  local r, ar;
  ar = array(int,1);
  r = iblines_raw(board, ar);
  return ar(1);
}


func ibfind(dev) {
  r = ibfind_raw(dev);
  if(ERR & r) {
    error, "ibfind";
  }
  return r;
}


extern ibonl;
/* PROTOTYPE  int ibonl(int ud, int onl);*/
/* DOCUMENT  int ibonl(int ud, int online);
Closes device (opposite ibfind)
Online arg: enable/disable
*/

extern ibpad;			
/* PROTOTYPE   int ibpad(int ud, int v);*/
/* DOCUMENT int ibpad(int dev, int address)
   Changes device address. 
*/

extern ibrpp_raw;			
/* PROTOTYPE   int ibrpp(int ud, char array  reply);*/
/* DOCUMENT int int ibrpp_raw(int ud, char array  reply);
   I do not know what it is doing and why I need it --- A. G.
*/


extern ibrsp_raw;
/* PROTOTYPE 
 int ibrsp(int ud, char  array spr);
 */

func ibrsp(ud) {
  /* DOCUMENT ibrsp(ud) returns status byte
     of device ud */
  a = array(char, 1);
  errcode = ibrsp_raw(ud, a);
  
  return a(1);
}

extern ibrsv;
/* PROTOTYPE 
 int ibrsv(int ud, int statusbyte)  */
/* DOCUMENT Changes serial poll responce of the specified GPIB board
   or device
 */

extern ibsad;
/* PROTOTYPE 
 int ibsad(int ud, int address ) */
/* DOCUMENT  int ibsad(int ud, int address ) 
   Assign/unassign secondary address of device
 */

extern ibsic;
/* PROTOTYPE 
 int ibsic(int board) */
/* DOCUMENT int ibsic(board_handle) 
Assert Interface Clear. Reinitialize GPIB system.  */

extern ibsre;
/* PROTOTYPE 
 int ibsre(int ud, int ren ) */
/* DOCUMENT  int ibsre(int ud, int ren ) 
   Asserts/unasserts remote enable line.
 */

extern ibtmo_raw;
/* PROTOTYPE
   int ibtmo(int ud, int v);
   */

func ibtmo(ud, v) {
  /* DOCUMENT ibtmo(ud) 
     set timeout for device ud */

  errcode = ibtmo_raw(ud, v);
  if(errcode & ERR) {
    error, "ibtmo";
  }
}

extern ibtrg;
/* PROTOTYPE  int ibtrg(int ud); */
/* DOCUMENT  int ibtrg(int ud);
 Triggers specified device
*/

extern ibwait;
/* PROTOTYPE int ibwait(int dev, int mask); */
/* DOCUMENT waits for some line to be asserted
   mask bits: the same as in ibsta.
*/

extern  ibwrt_raw;
/* PROTOTYPE
int ibwrt(int ud, string rd, long cnt);
*/

func ibwrt(ud, command) {
  /*DOCUMENT  ibwrt(ud, command)
   writes string to devise ud
   */
  l = strlen(command);
  r=ibwrt_raw(ud, command, l);
  if(ERR & r) {
    error, "ibwrt";
  }
  return r;
}





extern  ibrd_raw;
/* PROTOTYPE
int ibrd(int ud, char array rd, long cnt);
*/

func ibrd(ud, &answer,max_len) {
  /* DOCUMENT   ibrd(ud, &answer, max_len)
     Reads data from device ud */

  a = array(char, max_len);
  r = ibrd_raw(ud, a, max_len);
  if(ERR & r) {
    error, "ibrd";
  }
  answer=string(&a);
  return  r;
  
}

func ibread(ud, max_len) {
  /* DOCUMENT ibread is not compatible with 
     NatInst library, but is easier to use in Yorick */

  ibrd, ud, a, max_len;
  return a;
}

func ibrd_ascii_float(ud) {
  /* DOCUMENT reads 1 float number in ASCII from device*/
  local aa;
  aa = array(float,1);
  sread,ibread(ud,80), format="%g", aa;
  return aa(1);
}

extern convert_char_to_float; 
/* PROTOTYPE
   void convert_char_to_float(long Nfloat, char array c, float array a) 
*/

extern convert_char_to_short; 
/* PROTOTYPE
   void convert_char_to_short(long Nshort, char array c, short array a) 
*/

func ibrd_float(ud, N) {
  local a, r,f;
  /* DOCUMENT ibrd_float(ud,N) --- reads
     binary data from device ud
     Before using it, call ibeot to disable read termination on EOS
     */
  
  a = array(char, 4*N);
  f = array(float,N);
  r = ibrd_raw(ud, a, 4*N);
  if(ERR & r) {
    error, "ibrd_float";
  }
  convert_char_to_float,N, a, f;  
  return f;
}

func ibrd_short(ud, N) {
  local a, r,f;
  /* DOCUMENT ibrd_short(ud,N) --- reads
     binary data from device ud
     Before using it, call ibeot to disable read termination on EOS
     */
  
  a = array(char, 2*N);
  f = array(short,N);
  r = ibrd_raw(ud, a, 2*N);
  if(ERR & r) {
    error, "ibrd_short";
  }
  convert_char_to_short,N, a, f;  
  return f;
}


